from rich.console import Console
from rich.table import Table
from rich import box
import pandas as pd
from InquirerPy import inquirer
from InquirerPy.base.control import Choice
from letterboxd_stats import config
from ascii_magic import AsciiArt
from datetime import datetime

IMAGE_URL = "https://www.themoviedb.org/t/p/w600_and_h900_bestv2"


def select_value(values: list[str], message: str, default: str | None = None):
    value = inquirer.select(  # type: ignore
        message=message,
        choices=values,
        default=default or values[0],
    ).execute()
    return value


def select_movie_id(movies_info: pd.DataFrame) -> int:
    movie_id = inquirer.fuzzy(  # type: ignore
        message="Write movie id for more information",
        mandatory=False,
        max_height="25%",
        choices=[
            Choice(value=id, name=f"{id} - {title}") for id, title in zip(movies_info["Id"], movies_info["Title"])
        ],
        keybindings={"skip": [{"key": "escape"}]},
        validate=lambda result: result in movies_info["Id"].values,
        filter=lambda result: None if result is None else int(result),
        invalid_message="Input must be in the resulting IDs",
    ).execute()
    return movie_id


def select_search_result(results: list[str]) -> int:
    choices = [Choice(i, name=r) for i, r in enumerate(results)]
    result = inquirer.select(  # type: ignore
        message="Result of your search. Please select one",
        choices=choices,
        default=choices[0],
    ).execute()
    return result


def select_range(options: list[str]) -> list[str]:
    result = inquirer.rawlist(  # type: ignore
        message="Pick a desired value (or select 'all'). Use space to toggle your choices. CTRL+R to select all.",
        choices=options,
        default=options[0],
        multiselect=True,
        validate=lambda result: len(result) > 0,
    ).execute()
    return result


def select_movie(movie_df: pd.DataFrame) -> str:
    result = inquirer.fuzzy(  # type: ignore
        message="Select movie for more information",
        mandatory=False,
        max_height="25%",
        choices=[Choice(value=url, name=f"{title}") for url, title in zip(movie_df["Url"], movie_df["Title"])],
        keybindings={"skip": [{"key": "escape"}]},
        invalid_message="Input must be in the resulting IDs",
    ).execute()
    return result


def print_film(film, expand=True):
    grid = Table.grid(expand=expand, padding=1)
    grid.add_column(style="bold yellow")
    grid.add_column()
    for k, v in film.items():
        grid.add_row(str(k), str(v))
    console = Console()
    console.print(grid)


def render_table(df: pd.DataFrame, name: str):
    df_str = df.astype(str)
    table = Table(title=name, box=box.SIMPLE)
    for col in df_str.columns:
        table.add_column(col)
    for _, row in df_str.iterrows():
        table.add_row(*row)
    console = Console()
    console.print(table)


def download_poster(poster: str):
    if config["CLI"]["poster_columns"] > 0:
        art = AsciiArt.from_url(IMAGE_URL + poster)
        art.to_terminal(columns=int(config["CLI"]["poster_columns"]))


def _validate_date(s: str):
    try:
        datetime.strptime(s, "%Y-%m-%d")
    except ValueError:
        return False
    return True


def add_film_questions():
    print("Set all the infos for the film:\n")
    specify_date = inquirer.confirm(message="Specify date?").execute()  # type: ignore
    today = datetime.today().strftime("%Y-%m-%d")
    get_specified_date = inquirer.text(  # type: ignore
        message="Set viewing date:",
        default=today,
        validate=lambda d: _validate_date(d),
        invalid_message="Wrong date format",
    )
    specified_date = get_specified_date.execute() if specify_date else today
    stars = inquirer.number(  # type: ignore
        message="How many stars? (Only values from 0 to 5)",
        float_allowed=True,
        min_allowed=0.0,
        max_allowed=5.0,
        validate=lambda n: (2 * float(n)).is_integer(),
        invalid_message="Wrong value. Either an integer or a .5 float",
        replace_mode=True,
        filter=lambda n: int(2 * float(n)),
    ).execute()
    liked = inquirer.confirm(message="Did you like the movie?").execute()  # type: ignore
    review = inquirer.text(  # type: ignore
        message="Write a review. Press Enter for multiline.", multiline=True
    ).execute()
    contains_spoilers = False
    if len(review) > 0:
        contains_spoilers = inquirer.confirm(message="The review contains spoilers?").execute()  # type: ignore
    rewatch = inquirer.confirm(message="Have you seen this film before?").execute()  # type: ignore
    payload = {
        "specifiedDate": specify_date,
        "viewingDateStr": specified_date,
        "rating": stars,
        "liked": liked,
        "review": review,
        "containsSpoilers": contains_spoilers,
        "rewatch": rewatch,
    }
    if len(review.strip().rstrip("\n")) > 0 or specify_date:
        tags = inquirer.text(message="Add some comma separated tags. Or leave this empty.").execute()  # type: ignore
        payload["tag"] = (tags.split(",") if "," in tags else tags) if len(tags) > 0 else ""
    return payload



import os
from zipfile import ZipFile
from letterboxd_stats import config
from letterboxd_stats import cli
import requests
from lxml import html

URL = "https://letterboxd.com"
LOGIN_PAGE = URL + "/user/login.do"
DATA_PAGE = URL + "/data/export"
ADD_DIARY_URL = URL + "/s/save-diary-entry"
MOVIE_OPERATIONS = {
    "Add to diary": "add_film_diary",
    "Add to watchlist": "add_watchlist",
    "Remove from watchlist": "remove_watchlist",
}
OPERATIONS_URLS = {
    "search": lambda s: f"/search/films/{s}/?adult",
    "diary": lambda s: f"/csi/film/{s}/sidebar-user-actions/?esiAllowUser=true",
    "add_watchlist": lambda s: f"/film/{s}/add-to-watchlist/",
    "remove_watchlist": lambda s: f"/film/{s}/remove-from-watchlist/",
    "film_page": lambda s: f"/film/{s}",
}


class Downloader:
    def __init__(self):
        self.session = requests.Session()
        self.session.get(URL)

    def login(self):
        request_payload = {
            "username": config["Letterboxd"]["username"],
            "password": config["Letterboxd"]["password"],
            "__csrf": self.session.cookies.get("com.xk72.webparts.csrf"),
        }
        res = self.session.post(LOGIN_PAGE, data=request_payload)
        if res.json()["result"] != "success":
            raise ConnectionError("Impossible to login")

    def download_stats(self):
        res = self.session.get(DATA_PAGE)
        if res.status_code != 200 or "application/zip" not in res.headers["Content-Type"]:
            raise ConnectionError(f"Impossible to download data. Response headers:\n{res.headers}")
        print("Data download successful.")
        filename = res.headers["content-disposition"].split()[-1].split("=")[-1]
        path = os.path.expanduser(os.path.join(config["root_folder"], "static"))
        if not os.path.exists(path):
            os.makedirs(path)
        archive = os.path.join(path, filename)
        with open(archive, "wb") as f:
            f.write(res.content)
        with ZipFile(archive, "r") as zip:
            zip.extractall(path)
        os.remove(archive)

    def add_film_diary(self, title_url: str):
        payload = cli.add_film_questions()
        url = create_movie_url(title_url, "diary")
        res = self.session.get(url)
        if res.status_code != 200:
            raise ConnectionError("It's been impossible to retireve the Letterboxd page")
        movie_page = html.fromstring(res.text)
        letterboxd_film_id = movie_page.get_element_by_id("frm-sidebar-rating").get("data-film-id")
        payload["filmId"] = letterboxd_film_id
        payload["__csrf"] = self.session.cookies.get("com.xk72.webparts.csrf")
        res = self.session.post(ADD_DIARY_URL, data=payload)
        if not (res.status_code == 200 and res.json()["result"] is True):
            raise ConnectionError("Add diary request failed.")
        print("The movie was added to your diary.")

    def add_watchlist(self, title_url: str):
        url = create_movie_url(title_url, "add_watchlist")
        res = self.session.post(url, data={"__csrf": self.session.cookies.get("com.xk72.webparts.csrf")})
        if not (res.status_code == 200 and res.json()["result"] is True):
            raise ConnectionError("Add diary request failed.")
        print("Added to your watchlist.")

    def remove_watchlist(self, title_url: str):
        url = create_movie_url(title_url, "remove_watchlist")
        res = self.session.post(url, data={"__csrf": self.session.cookies.get("com.xk72.webparts.csrf")})
        if not (res.status_code == 200 and res.json()["result"] is True):
            raise ConnectionError("Add diary request failed.")
        print("Removed to your watchlist.")

    def perform_operation(self, answer: str, link: str):
        getattr(self, MOVIE_OPERATIONS[answer])(link)


def create_movie_url(title: str, operation: str):
    url = URL + OPERATIONS_URLS[operation](title)
    return url


def get_tmdb_id(link: str, is_diary: bool):
    res = requests.get(link)
    movie_page = html.fromstring(res.text)
    if is_diary:
        title_link = movie_page.xpath("//span[@class='film-title-wrapper']/a")
        if len(title_link) > 0:
            movie_link = title_link[0]
            movie_url = URL + movie_link.get("href")
            movie_page = html.fromstring(requests.get(movie_url).text)
    tmdb_link = movie_page.xpath("//a[@data-track-action='TMDb']")
    if len(tmdb_link) > 0:
        id = tmdb_link[0].get("href").split("/")[-2]
        return int(id)
    return None


def select_optional_operation():
    return cli.select_value(["Exit"] + list(MOVIE_OPERATIONS.keys()), "Select operation:")


def search_film(title: str, allow_selection=False):
    search_url = URL + OPERATIONS_URLS["search"](title)
    res = requests.get(search_url)
    if res.status_code != 200:
        raise ConnectionError("It's been impossible to retireve the Letterboxd page")
    search_page = html.fromstring(res.text)
    if allow_selection:
        movie_list = search_page.xpath("//div[@class='film-detail-content']")
        if len(movie_list) == 0:
            raise ValueError(f"No film found with search query {title}")
        titles = [movie.xpath("./h2/span/a")[0].text.rstrip() for movie in movie_list]
        years = [
            f"({year[0].text}) " if len(year := movie.xpath("./h2/span//small/a")) > 0 else "" for movie in movie_list
        ]
        directors = [movie.xpath("./p/a")[0].text for movie in movie_list]
        links = [movie.xpath("./h2/span/a")[0].get("href") for movie in movie_list]
        title_years_directors_links = {
            f"{title} {year}- {director}": link for title, year, director, link in zip(titles, years, directors, links)
        }
        selected_film = cli.select_value(list(title_years_directors_links.keys()), "Select your film")
        title_url = title_years_directors_links[selected_film].split("/")[-2]
    else:
        title_url = search_page.xpath("//span[@class='film-title-wrapper']/a")[0].get("href").split("/")[-2]
    return title_url



from letterboxd_stats import tmdb
from letterboxd_stats import data
from letterboxd_stats import web_scraper as ws
import os
from letterboxd_stats import args, config

DATA_FILES = {"Watchlist": "watchlist.csv", "Diary": "diary.csv", "Ratings": "ratings.csv", "Lists": "lists"}


def try_command(command, args):
    try:
        command(*args)
    except Exception as e:
        print(e)


def check_path(path: str):
    if not os.path.exists(path):
        raise FileNotFoundError(
            f"No Letterboxd data was found in {path}. Make sure the path is correct or run -d to download your data"
        )


def download_data():
    downloader = ws.Downloader()
    downloader.login()
    downloader.download_stats()


def get_movie_detail_from_url(letterboxd_url, is_diary=False):
    if letterboxd_url is not None:
        id = ws.get_tmdb_id(letterboxd_url, is_diary)
        if id is not None:
            tmdb.get_movie_detail(id, letterboxd_url)


def search_person(args_search: str):
    df, name = tmdb.get_person(args_search)
    path = os.path.expanduser(os.path.join(config["root_folder"], "static", "watched.csv"))
    check_path(path)
    movie_id, search_film = data.read_watched_films(df, path, name)
    letterboxd_url = ws.search_film(search_film)
    if movie_id is not None:
        tmdb.get_movie_detail(movie_id, ws.create_movie_url(letterboxd_url, "film_page"))


def search_film(args_search_film: str):
    film_url = ws.search_film(args_search_film, True)
    film_page_url = ws.create_movie_url(film_url, "film_page")
    tmdb.get_movie_detail(ws.get_tmdb_id(film_page_url, False), film_page_url)  # type: ignore
    answer = ws.select_optional_operation()
    if answer != "Exit":
        downloader = ws.Downloader()
        downloader.login()
        downloader.perform_operation(answer, film_url)


def get_data(args_limit, args_ascending, data_type):
    path = os.path.expanduser(os.path.join(config["root_folder"], "static", DATA_FILES[data_type]))
    check_path(path)
    letterboxd_url = (
        data.open_file(data_type, path, args_limit, args_ascending)
        if data_type != "Lists"
        else data.open_list(path, args_limit, args_ascending)
    )
    get_movie_detail_from_url(letterboxd_url, data_type == "Diary")


def main():
    if args.download:
        try_command(download_data, ())
    if args.search:
        try_command(search_person, (args.search,))
    if args.search_film:
        try_command(search_film, (args.search_film,))
    if args.watchlist:
        try_command(get_data, (args.limit, config["CLI"]["ascending"], "Watchlist"))
    if args.diary:
        try_command(get_data, (args.limit, config["CLI"]["ascending"], "Diary"))
    if args.ratings:
        try_command(get_data, (args.limit, config["CLI"]["ascending"], "Ratings"))
    if args.lists:
        try_command(get_data, (args.limit, config["CLI"]["ascending"], "Lists"))


if __name__ == "__main__":
    main()
